;|
Funktion: MS_GETINI
Lisp-Funktion die (fast) analog zu DOS_GETINI
aus dem DosLib von Robert McNeel arbeitet.

Erklrung hierzu:
----------------------------------------------------------------

Funktion gibt einen String aus einer Windows-typischen
Initialisierungs (*.ini) Datei zurck.
Die INI Datei mu folgende Form aufweisen:

...
[section] 
entry=string
entry2=string
...

Kommentare in der INI werden durch ein Semikolon gekennzeichnet:

...
; Dies ist ein Kommentar
...

Syntax:
(ms_getini section entry filename)

Parameter:
section 
 Die Sektion die entry enthlt

entry
 Der Eintrag der mit dem zurckzugebenen String assoziiert ist.

filename
 Der Dateiname des Ini-Files.
 Pfad und Name sind erforderlich !!!

Anmerkung: Die Parameter sind nicht "Case-Sensitive", Gro/Klein-Schreibung spielt keine Rolle.

Rckgabewerte:
Wenn section als nil bergeben wird, ist der Rckgabewert eine Liste der sections.

Wenn entry als nil bergeben wird, ist der Rckgabewert eine Liste der entries.

Wenn entry gefunden wurde, wird der assoziierte String zurckgegeben.

Anderenfalls, oder beim Auftreten eines Fehlers wird nil zurckgegeben.

Beispiele:
Befehl: (ms_getini nil nil "c:\\program files\\autocad 2002\\acad2002.cfg")
-> ("AppData" "Version" "AutoCAD" "Digitizer/Driver" "Editor" "Menu") 

Befehl: (ms_getini "version" nil "c:\\program files\\autocad 2002\\acad2002.cfg")
-> ("Platform" "FileVersion" "ExecutableDate&Time") 

Befehl: (ms_getini "version" "platform" "c:\\program files\\autocad 2002\\acad2002.cfg")
-> "Microsoft Windows"
|;
(defun MS_GETINI (SECTION ENTRY FILENAME / ASS INI-LST)
  (if (setq INI-LST (INI->LST FILENAME))
    (cond
      ((not SECTION)
       (REMOVE-COMMENT INI-LST)
      )
      ((not ENTRY)
       (if (setq ASS (ms-assoc SECTION INI-LST))
	 (REMOVE-COMMENT (cadr ASS))
       ) ;_ end of if
      )
      (t
       (if (setq ASS (ms-assoc SECTION INI-LST))
	 (if (setq ASS (ms-assoc ENTRY (cadr ASS)))
	   (cdr ASS)
	 ) ;_ end of if
       ) ;_ end of if
      )
    ) ;_ end of cond
  ) ;_ end of if
) ;_ end of defun

;|
Funktion: MS_SETINI
Lisp-Funktion die (fast) analog zu DOS_SETINI
aus dem DosLib von Robert McNeel arbeitet.

Erklrung hierzu:
----------------------------------------------------------------

Funktion kopiert einen String an eine bestimmte Stelle
in eine Windows-typische Initialisierungs (*.ini) Datei.
Die INI Datei mu folgende Form aufweisen:

...
[section] 
entry=string
entry2=string
...

Kommentare in der INI werden durch ein Semikolon gekennzeichnet:

...
; Dies ist ein Kommentar
...

Syntax 
(ms_setini section entry string filename)

Parameter:
section
 Die section in die der String kopiert werden soll.
 Wenn section nicht in der Datei existiert, wird sie erzeugt.

entry
 Der Name von entry mit dem der String assoziiert ist.
 Wenn entry NICHT in der definierten Sektion existiert wird er erzeugt.
 Wenn entry als nil bergeben wird, wird die gesamte section, inkl. ALLER entries gelscht.

string
 Der string der in die Datei geschrieben werden soll.
 Wenn string als nil bergeben wird, lscht dies die Zeile die durch entry definiert wird.

filename
 Pfad und Name der INI-Datei.
 Volle Pfadangabe ist erforderlich !!
 Wenn filename nicht existiert, erzeugt die Funktion eine neue Datei (der Pfad mu aber bereits existieren !!!). 


Anmerkung: Die Parameter sind nicht "Case-Sensitive", Gro/Klein-Schreibung spielt keine Rolle.

Rckgabewert:
Die volle Pfadangabe fr die INI-Datei.

nil bei auftreten eines Fehlers.

Beispiele:
Befehl: (ms_setini "section1" "entry1" "string1" "C:\\temp\\filename.ini")
-> "C:\\temp\\filename.ini"

Befehl: (ms_setini "section1" "entry2" "string2" "C:\\temp\\filename.ini")
-> "C:\\temp\\filename.ini"

Befehl: (ms_setini "section2" "entry1" "string1" "C:\\temp\\filename.ini")
-> "C:\\temp\\filename.ini"

Wenn section nicht existiert = erzeugen der section
Wenn entry nil ist = lschen der gesamten section
Wenn value nil ist = lschen von entry
Wenn filename nicht existiert = erzeugen von filename, pfad mu aber bestehen!
|;
(defun MS_SETINI (SECTION ENTRY VALUE FILENAME / INI-LST ORG-INI-LST SUB-LST SUB-VALUE FILE-DESC)
  (if (VERIFY-OR-CREATE-FILE FILENAME) ;_ Filename verifizieren oder erzeugen der Datei
    (if (setq INI-LST (INI->LST FILENAME))
      (progn
        (setq ORG-INI-LST INI-LST) ;_ Fr abschlieenden Vergleich, ob berhaupt was gendert wurde...
        (if SECTION ;_ Wenn section nil ist, sollte nix passieren
          (cond
            ((not ENTRY)
             (setq INI-LST (vl-remove-if
                             '(lambda (X)
                                (equal X (MS-ASSOC SECTION INI-LST))
                              ) ;_ end of lambda
                             INI-LST
                           ) ;_ end of vl-remove-if
             ) ;_ end of setq
            )
            ((not VALUE)
             (if (setq SUB-LST (MS-ASSOC SECTION INI-LST))
               (setq INI-LST
                      (subst (list
                               (car SUB-LST)
                               (vl-remove-if
                                 '(lambda (X)
                                    (equal (strcase (car X)) (strcase ENTRY))
                                  ) ;_ end of lambda
                                 (cadr SUB-LST)
                               ) ;_ end of vl-remove-if
                             ) ;_ end of list
                             SUB-LST
                             INI-LST
                      ) ;_ end of subst
               ) ;_ end of setq
             ) ;_ end of if
            )
            (t
;;; Hier geht's weita !!!
;;; Handling fr's nicht vorhandensein der section
;;; und Handling fr Manipulation vorhandener Werte
             (if (setq SUB-LST (MS-ASSOC SECTION INI-LST)) ;_ section existiert?
               (if (setq SUB-VALUE (MS-ASSOC ENTRY (cadr SUB-LST))) ;_entry in section existiert?
                 (setq INI-LST (subst (list (car SUB-LST)
                                            (subst (cons ENTRY VALUE)
                                                   SUB-VALUE
                                                   (cadr SUB-LST)
                                            ) ;_ end of subst
                                      ) ;_ end of list
                                      SUB-LST
                                      INI-LST
                               ) ;_ end of subst
                 ) ;_ end of setq
                 ;;_ entry existiert nicht:
                 (setq
                   INI-LST (subst
                             (list
                               (car SUB-LST)
                               (reverse (cons (cons ENTRY VALUE)
                                              (reverse (cadr SUB-LST))
                                        ) ;_ end of cons
                               ) ;_ end of reverse
                             ) ;_ end of list
                             SUB-LST
                             INI-LST
                           ) ;_ end of subst
                 ) ;_ end of setq
               ) ;_ end of if
               ;; section existiert nicht:
               (setq INI-LST
                      (reverse
                        (cons (list SECTION (list (cons ENTRY VALUE)))
                              (reverse INI-LST)
                        ) ;_ end of cons
                      ) ;_ end of reverse
               ) ;_ end of setq
             ) ;_ end of if
            )
          ) ;_ end of cond

        ) ;_ end of if
        (if (not (equal INI-LST ORG-INI-LST)) ;_ Es wurden Manipulationen an der Liste vorgenommen...
          (if (setq FILE-DESC (open FILENAME "w")) ;_ Kann in die Datei geschrieben werden?
            (progn
              (foreach ELEM INI-LST
                (if (not (equal (car ELEM) "comment"))
                  (write-line (strcat "[" (car ELEM) "]") FILE-DESC)
                ) ;_ end of if
                (foreach SUB-ELEM (cadr ELEM)
                  (if (equal (car SUB-ELEM) "comment")
                    (write-line (cdr SUB-ELEM) FILE-DESC)
                    (write-line
                      (strcat (car SUB-ELEM) "=" (cdr SUB-ELEM))
                      FILE-DESC
                    ) ;_ end of write-line
                  ) ;_ end of if
                ) ;_ end of foreach
              ) ;_ end of foreach
              (close file-desc)
              filename ;_ Rckgabewert der Funktion wenn alles OK war
            ) ;_ end of progn
          ) ;_ end of if
        ) ;_ end of if
      ) ;_ end of progn
    ) ;_ end of if
  ) ;_ end of if
) ;_ end of defun

;;; Prft das Vorhandensein einer Datei und deren Pfad
;;; Wenn die Datei nicht existiert, der Pfad aber gltig ist,
;;; wird die entsprechende Datei erzeugt.
(defun VERIFY-OR-CREATE-FILE (FILENAME / FILE-DESC)
  (if (not (findfile FILENAME))
    (if (vl-file-directory-p (vl-filename-directory FILENAME))
      (if (setq FILE-DESC (open FILENAME "w"))
        (progn
          (close FILE-DESC)
          FILENAME
        ) ;_ end of progn
      ) ;_ end of if
    ) ;_ end of if
    (if (MS_WRITE-P FILENAME)
      filename
      )
  ) ;_ end of if
) ;_ end of defun

;;; berprft, ob eine Datei Schreibgeschtzt ist
(defun MS_WRITE-P (FILENAME / FILE-DESC)
  (if (setq FILE-DESC (open FILENAME "a"))
    (progn
      (close FILE-DESC)
      FILENAME
    ) ;_ end of progn
  ) ;_ end of if
) ;_ end of defun  

;;; Nomen est Omen !
(defun REMOVE-COMMENT (LST /)
  (vl-remove-if
    '(lambda (X)
       (equal X "comment")
     ) ;_ end of lambda
    (mapcar
      '(lambda (X)
	 (car X)
       ) ;_ end of lambda
      LST
    ) ;_ end of mapcar
  ) ;_ end of vl-remove-if
) ;_ end of defun

;;; Ersatz fr Assoc. Assoc ist in diesem Fall case-sensitive
;;; ms-assoc nicht. Kontrolle fr doppelte Eintrge fehlt hier
;;; aber assoc gibt ja auch immer nur den ersten zurck...
(defun MS-ASSOC	(STR LST / retval)
  (if (setq
	RETVAL (vl-remove-if-not
		 '(lambda (X) (equal (strcase (car X)) (strcase STR)))
		 LST
	       ) ;_ end of vl-remove-if-not
      ) ;_ end of setq
    (car RETVAL)
  ) ;_ end of if
) ;_ end of defun

;;; Erzeugt aus 'm ini-File 'ne assocable List
;;; Format:
;;; (("section" ((key1 . value)(key2 . value)))("section2" . ((key1 . value)(key2 . value))))
;;; Fr etwaige fhrende Kommentare (vor dem ersten "[section]" Eintrag) gibt es 'ne Dummy-Section.
;;; Diese Dummy-Section heit wie auch die Dummy-Keys (fr Kommentare innerhalb der Sections)
;;; [comment]
(defun INI->LST (FILENAME   /          BASE-LST   COMMENT    FILE-DESC
                 RETVAL     SECTION    SUB-LST    TOKEN      ZEILE
                )
  (setq FILE-DESC (open FILENAME "r")
        SECTION   "[comment]"
        TOKEN     "="
        COMMENT   ";"
  ) ;_ end of setq
  (if FILE-DESC
    (progn
      (while (= (type (setq ZEILE (read-line FILE-DESC))) 'STR)
        (setq BASE-LST (cons (vl-string-trim " " ZEILE) BASE-LST))
      ) ;_ end of while
      (close FILE-DESC)
      (if (not BASE-LST) ;_ Datei enthlt KEINERLEI Zeilen (z.B. Neu erzeugt)
        (setq RETVAL (list (list "comment"))) ;_ Dummy-Liste als Rckgabewert
        (progn
          (setq BASE-LST (reverse BASE-LST))
          (foreach ZEILE BASE-LST
            (cond
              ((or (equal (substr ZEILE 1 1) COMMENT) ;_ Ist die Zeile Kommentar?
                   (equal ZEILE "") ;_ Ist die Zeile Leer?
               ) ;_ end of or
               (setq SUB-LST (cons (cons "comment" ZEILE) SUB-LST))
              )
              ((wcmatch ZEILE "`[*`]") ;_ Ist die Zeile der Beginn einer neuen section?
               (setq SUB-LST NIL
                     SECTION ZEILE
               ) ;_ end of setq
              )
              (t
               (setq SUB-LST (cons (KEY->LST ZEILE TOKEN) SUB-LST))
              )
            ) ;_ end of cond
            (if (not (assoc SECTION RETVAL))
              (setq RETVAL (cons (list SECTION) RETVAL))
              (setq RETVAL (subst (list SECTION (reverse SUB-LST))
                                  (assoc SECTION RETVAL)
                                  RETVAL
                           ) ;_ end of subst
              ) ;_ end of setq
            ) ;_ end of if
          ) ;_ end of foreach
          (setq RETVAL
                 (reverse
                   (mapcar
                     '(lambda (X)
                        (list
                          (vl-string-right-trim
                            "]"
                            (vl-string-left-trim
                              "["
                              (car X)
                            ) ;_ end of vl-string-left-trim
                          ) ;_ end of vl-string-right-trim
                          (cadr X)
                        ) ;_ end of list
                      ) ;_ end of lambda
                     RETVAL
                   ) ;_ end of mapcar
                 ) ;_ end of reverse
          ) ;_ end of setq
        ) ;_ end of progn
      ) ;_ end of if
    ) ;_ end of progn
  ) ;_ end of if
  RETVAL
) ;_ end of defun

;;; Erzeugt aus der Ini-File typischen Key=Value Struktur
;;; eine cons-Liste ("key" . "value")
(defun KEY->LST	(STR TOKEN / POSITION)
  (setq POSITION (vl-string-search TOKEN STR))
  (cons (substr STR 1 POSITION) (substr STR (+ POSITION 2)))
) ;_ end of defun

;_ EOF
(princ)

